library(uoptr)
library(adr)

myuopt <- uoptCreate()
luopt <- myuopt$uopt

obj <- function(x) {
    N <- length(x)
    sum((1 - x[-N])^2 + 100 * (x[-1] - x[-N]^2)^2)
}

N <- 20

v0  <- array(rnorm(1:N)*10, c(1, N))

J0 <- obj(v0)

opts <- list(maxit=800, tolObjAbs=1e-7, sdirs=0x18,
             trace=0x3, saveplots=1)

vs <- luopt(v0,
           f = obj,
           g = function(x)
               adrDiffRev(obj, list(x)),
           H = function(x)
             adrHessRev(obj, list(x)),
           Hv = NULL,
           s = function(x, status) {
               uoptShowState(status)
             if ('st' %in% names(status)) {
              if (status$st != 0) {
                cat(sprintf('Non-0 status\n'))
              }
             }
               0
           },
           options = opts)

#save(myuopt, file=sprintf('myuopt-Hess-%d.Rdat', opts$sdirs))
r <- uoptReport(myuopt)
file.copy(r, 'report-rosenN.html', overwrite=TRUE)
#m <- uoptMovie(myuopt)
