import numpy as np

np.set_printoptions(linewidth=125)

def rosenNs(x):
    N = x.size
    return sum((1 - x[0:N-1])**2 + 100 * (x[1:N] - x[0:N-1]**2)**2)

def rosenN(x):
    N = x.size
    rterms1 = (1 - x[0:N-1])
    rterms2 = 10 * (x[1:N] - x[0:N-1]**2)
    r = np.hstack([rterms1, rterms2])
    return r

def rosenNObj(x, y, *args):
    y[:] = rosenN(x)

def rosenNJac(x, y, J, *args):
    rosenNObj(x, y)
    N = x.size
    yblk = N-1
    # rterms1 = (1 - x[0:N-1])
    J[0:yblk,:] = -np.eye(yblk, x.size)
    # rterms2 = 10 * (x[1:N] - x[0:N-1]**2)
    J[yblk:2*yblk,:] = np.diag(np.full(yblk,10), 1)[0:yblk,:]
    J[yblk:2*yblk,:] += np.diag(10 * -2 * x)[0:yblk,:]

def rosenNJacv(x, y, v, Jv, *args):
    rosenNObj(x, y)
    N = x.size
    yblk = N-1
    x = x.reshape(N)
    v = v.reshape(N)
    # rterms1 = (1 - x[0:N-1])
    Jv[0:yblk,0] = -1 * v[0:yblk]
    # rterms2 = 10 * (x[1:N] - x[0:N-1]**2)
    Jv[yblk:2*yblk,0] = 10 * (v[1:N] - 2 * x[0:yblk]*v[0:yblk])
    # J = np.ndarray((y.size, x.size), order='F')
    # rosenNJac(x, y, J)
    # Jv[:] = np.matmul(J, v)

def rosenNJacTv(x, y, v, Jv, *args):
    J = np.ndarray((y.size, x.size), order='F')
    rosenNJac(x, y, J)
    Jv[:] = np.matmul(v, J)

def rosenOutput(x, y, status, *args):
    maxLen = max(14, max([len(f)+1+len(status[f]) for f in status]))
    print("".join([("%%- %ds" % (maxLen+2,)) %
                   ("%s=%s" % (f, status[f])) for f in status]))
