import numpy as np


a = 2;
b = 100;


def fhandle(x, J, userdata):

    # Rosenbrock function:
    # (a - x[[1]])^2 + b * (x[[2]] - x[[1]]^2)^2

    r = (a - x[0])**2 + b * (x[1] - x[0]**2)**2;

    J[0] = r
    return 0


def ghandle(x, J, g, userdata):

    r = (a - x[0])**2 + b * (x[1] - x[0]**2)**2;

    # first derivative of Rosenbrock function:
    ## (a - x1)^2 + b * (x2 - x1^2)^2
    ## [-2 * (a - x1) - b * 2 * (x2 - x1^2) * 2 * x1, b * 2 * (x2 - x1^2)]

    dr1 = -2 * (a - x[0]) - b * 2 * (x[1] - x[0]**2) * 2 * x[0];
    dr2 = b * 2 * (x[1] - x[0]**2);

    J[0] = r

    g[0] = dr1
    g[1] = dr2
    return 0


def Hhandle(x, J, g, H, userdata):

    r = (a - x[0])**2 + b * (x[1] - x[0]**2)**2;

    dr1 = -2 * (a - x[0]) - b * 2 * (x[1] - x[0]**2) * 2 * x[0];
    dr2 = b * 2 * (x[1] - x[0]**2);

    # second derivative of Rosenbrock function:
    ## (a - x1)^2 + b * (x2 - x1^2)^2
    ## [-2 * (a - x1) - b * 2 * (x2 - x1^2) * 2 * x1, b * 2 * (x2 - x1^2)]

    dr1_dr1 = 2 + b * 2 * 2 * x[0] * 2 * x[0] - b * 2 * (x[1] - x[0]**2) * 2;
    dr1_dr2 = - b * 2 * 2 * x[0];
    dr2_dr1 = -b * 2 * 2 * x[0];
    dr2_dr2 = b * 2;

    J[0] = r

    g[0] = dr1
    g[1] = dr2

    H[0][0] = dr1_dr1
    H[0][1] = dr1_dr2

    H[1][0] = dr2_dr1
    H[1][1] = dr2_dr2

    return 0


def Hvhandle(x, J, dx, g, Hv, userdata):

    n = np.size(x)
    ndx = dx.shape[1]

    r = (a - x[0])**2 + b * (x[1] - x[0]**2)**2;

    dr1 = -2 * (a - x[0]) - b * 2 * (x[1] - x[0]**2) * 2 * x[0];
    dr2 = b * 2 * (x[1] - x[0]**2);

    # second derivative of Rosenbrock function:
    ## (a - x1)^2 + b * (x2 - x1^2)^2
    ## [-2 * (a - x1) - b * 2 * (x2 - x1^2) * 2 * x1, b * 2 * (x2 - x1^2)]

    dr1_dr1 = 2 + b * 2 * 2 * x[0] * 2 * x[0] - b * 2 * (x[1] - x[0]**2) * 2;
    dr1_dr2 = - b * 2 * 2 * x[0];
    dr2_dr1 = -b * 2 * 2 * x[0];
    dr2_dr2 = b * 2;

    J[0] = r

    g[0] = dr1
    g[1] = dr2

    H = np.zeros((2,2))
    H[0][0] = dr1_dr1;
    H[0][1] = dr1_dr2;

    H[1][0] = dr2_dr1;
    H[1][1] = dr2_dr2;

    Hv[:] = np.matmul(H, dx)

    return 0


def ohandle(x, y, status, userdata):
    print(x, y)
    print(status)
    return 0
