import numpy as np
import random
import math

class Sphere3DDataset:
    def __init__(self, M):
        self.M = M
        self.N = 3
        self.samples = np.ndarray((self.M,self.N))
        self.errors = []
        self.opts = {}

    def initSamples(self, R0, P0):
        for i in range(self.M):
            theta = random.random()
            phi = random.random()
            Pi = [ R0*math.cos(phi)*math.cos(theta) + P0[0],
                                  R0*math.sin(phi)*math.cos(theta) + P0[1],
                                  R0*math.sin(theta) + P0[2],
                                 ]
            self.samples[i,:] = Pi

    def dists(self, R, P):
        res = self.samples - P
        ds = np.sqrt(np.sum(res * res, 1))
        ds -= R
        return ds

    def ddists(self, R, P, Jac):
        res = self.samples - P
        ds = np.sqrt(np.sum(res * res, 1))
        dd_dx = -res / np.reshape(ds, (self.M,1))
        ds -= R
        Jac[:,0] = -1
        Jac[:,1:4] = dd_dx
        return ds

def unpack(x):
    return x[0], x[1:4]

def fhandle(x, y, sphere3d):
    R, P = unpack(x)
    y[:] = sphere3d.dists(R, P)
    return 0

def Jhandle(x, y, J, sphere3d):
    R, P = unpack(x)
    y[:] = sphere3d.ddists(R, P, J)
    return 0

def Jvhandle(x, y, v, Jv, sphere3d):
    R, P = unpack(x)
    J = np.ndarray((sphere3d.M,4))
#    print(v.shape, Jv.shape)
#    print(v.flags, Jv.flags)
    y[:] = sphere3d.ddists(R, P, J)
    Jv[:] = np.matmul(J, v)
#    print(Jv.T)
    return 0

def JTvhandle(x, y, v, Jv, sphere3d):
    R, P = unpack(x)
    J = np.ndarray((sphere3d.M,4))
#    print(v.shape, Jv.shape)
#    print(v.flags, Jv.flags)
    y[:] = sphere3d.ddists(R, P, J)
    Jv[:] = np.matmul(v, J)
#    print(Jv)
    return 0

def ohandle(x, y, status, sphere3d):
    print(x)
    R, P = unpack(x)
    y2 = sphere3d.dists(R, P)
    #print(np.linalg.norm(y))
    #print(np.linalg.norm(y2))
    #print(np.linalg.norm(y - y2))
    if 'J' in status:
        errJ = abs(float(status['J']) - np.linalg.norm(y2))
        if errJ > 1e-5:
            errmsg = 'Error: current objective reported is not verifyable: %g' % errJ
            print(errmsg)
            sphere3d.errors += [BaseException(errmsg)]
    erry = np.linalg.norm(y - y2)
    if erry > 1e-14:
        errmsg = 'Error: current result reported is not verifyable: %g' % erry
        print(errmsg)
        sphere3d.errors += [BaseException(errmsg)]
    print(status)
    res = 0
    if 'it' in status and 'Er' in status:
        # iteration case
        if float(status['Er']) <= 0:
            print('Non-advancement step Er = %g' % float(status['Er']))
            res = 2 # stop iteration with failure, exit code 1
        if int(status['it']) > 42:
            if R > 200:
                print('Runaway case, R = %g' % R)
                res = 2 # stop iteration with failure, exit code 1
            if float(status['J']) < 1e-14:
                print('Objective is small enough, exit success')
                res = 1 # -> stop iteration with success, exit code 0
    if 'status' in status and int(status['status']) != 0:
        if status['exit'] != 'stop code set':
            if status['exit'] == 'search failure' and float(status['J']) < 2e-13:
                # OK
                print('search failure is OK when J is small')
                res = 1 # -> exit code 0
            else:
                print('Error other than runaway case detected at exit')
                res = 2 # -> exit code 2
        else:
            print('Stop code set because of runaway case, R = %g' % R)
            res = 2

    if 'status' in status and int(status['status']) == 0:
        if 'checkres' in sphere3d.opts:
            if not sphere3d.opts['checkres'](status):
                print('exit test condition %s failed' % (sphere3d.opts['checkres']))
                print('options', sphere3d.opts)
                res = 2
    return res
