<?xml version="1.0"?>
<!--
  Author: Johannes Willkomm <jwillkomm@ai-and-it.de>
-->
<xsl:stylesheet version="1.0"
                xmlns:xhtml="http://www.w3.org/1999/xhtml"
                xmlns="http://www.w3.org/1999/xhtml"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:param name="template-doc-uri" select="''"/>
  <xsl:param name="image-doc-uri" select="''"/>
  <xsl:param name="base" select="''"/>

  <xsl:param name="template-doc" select="document($template-doc-uri, /)"/>
  <xsl:param name="image-doc" select="document($image-doc-uri, /)"/>

  <xsl:variable name="xhtmlns" select="'http://www.w3.org/1999/xhtml'"/>

  <xsl:variable name="root" select="/"/>

  <xsl:output method="xml"/>

  <xsl:template match="/">
    <xsl:if test="not($template-doc)">
      <xsl:message>Could not load template document '<xsl:value-of select="$template-doc-uri"/>'
</xsl:message>
    </xsl:if>
    <xsl:if test="not($image-doc)">
      <xsl:message>Could not load image data document '<xsl:value-of select="$image-doc-uri"/>'
</xsl:message>
    </xsl:if>
    <xsl:apply-templates select="$template-doc/*"/>
  </xsl:template>

  <xsl:template match="xhtml:*">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="xhtml:head">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates select="$root/*/head/style" mode="copy-to-xhtml"/>
      <xsl:apply-templates select="$root/*/head/script" mode="copy-to-xhtml"/>
      <xsl:apply-templates select="$root/*/head/meta" mode="copy-to-xhtml"/>
      <xsl:apply-templates select="$root/*/head/title" mode="copy-to-xhtml"/>
      <xsl:apply-templates select="$root/*/head/link" mode="copy-to-xhtml"/>
      <xsl:apply-templates/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="*" mode="copy-to-xhtml">
    <xsl:element name="{local-name()}" namespace="{$xhtmlns}">
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates mode="copy-to-xhtml"/>
    </xsl:element>
  </xsl:template>

  <xsl:template match="xhtml:div[@data-report-here='1']">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:apply-templates select="$root/*/report"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="report">
    <div>
      <xsl:copy-of select="@*"/>
      <h1><xsl:apply-templates select="title"/></h1>
      <xsl:apply-templates select="*[position()>1]"/>
    </div>
  </xsl:template>

  <xsl:template match="section">
    <xsl:variable name="lev" select="count(ancestor::section)"/>
    <div class="sec-{$lev+1}">
      <xsl:element name="h{$lev+2}">
        <xsl:apply-templates select="title"/>
      </xsl:element>
      <xsl:apply-templates select="*[position()>1]"/>
    </div>
  </xsl:template>

  <xsl:template match="para">
    <p>
      <xsl:apply-templates/>
    </p>
  </xsl:template>

  <xsl:template match="div">
    <div>
      <xsl:apply-templates/>
    </div>
  </xsl:template>

  <xsl:template match="table">
    <div class="table">
      <p>
        <span><b>Table <xsl:value-of select="count(preceding::table)+1"/></b>: <xsl:apply-templates select="caption"/></span>
      </p>
      <xsl:apply-templates select="data"/>
    </div>
  </xsl:template>

  <xsl:template match="caption">
    <span class="caption" style="font-style: italic">
      <xsl:apply-templates/>
    </span>
  </xsl:template>

  <xsl:template match="table/data">
    <table>
      <tr>
        <th>Name</th>
        <th>Value</th>
      </tr>
      <xsl:for-each select="*">
        <tr>
          <td>
            <xsl:value-of select="name()"/>
          </td>
          <td>
            <xsl:apply-templates/>
          </td>
        </tr>
      </xsl:for-each>
    </table>
  </xsl:template>

  <xsl:template match="figure">
    <div class="figure">
      <xsl:apply-templates select="graph"/>
      <p>
        <span><b>Figure <xsl:value-of select="count(preceding::figure)+1"/></b>: <xsl:apply-templates select="caption"/></span>
      </p>
    </div>
  </xsl:template>

  <xsl:template match="graph">
    <xsl:variable name="skey" select="."/>
    <xsl:variable name="sres" select="$image-doc//data.item[id = $skey]"/>
    <xsl:if test="not($sres)">
      <xsl:message>Could not find image data for id '<xsl:value-of select="$skey"/>' (from <xsl:value-of
        select="count($image-doc//data.item)"/> avail.)</xsl:message>
    </xsl:if>
    <img src="data:image/png;base64,{$sres/data}"/>
  </xsl:template>

  <xsl:template match="graph-">
    <img src="{.}"/>
  </xsl:template>

</xsl:stylesheet>
